@@ -1,17 +1,43 @@
 package Set::Scalar::Base;
 
 use strict;
-local $^W = 1;
+# local $^W = 1;
 
 require Exporter;
 
-use vars qw(@ISA @EXPORT_OK);
+use vars qw($VERSION @ISA @EXPORT_OK);
 
+$VERSION = '1.29';
 @ISA = qw(Exporter);
 
-use UNIVERSAL 'isa';
-
-use Scalar::Util qw(refaddr blessed);
+BEGIN {
+    eval 'require Scalar::Util';
+    unless ($@) {
+	import Scalar::Util qw(blessed refaddr);
+    } else {
+	# Use the pure Perl emulations (directly snagged from Scalar::Util).
+	eval 'sub UNIVERSAL::a_sub_not_likely_to_be_here { ref($_[0]) }';
+	*blessed = sub ($) {
+	    local($@, $SIG{__DIE__}, $SIG{__WARN__});
+	    length(ref($_[0]))
+		? eval { $_[0]->a_sub_not_likely_to_be_here }
+	    : undef
+	};
+        *refaddr = sub ($) {
+	    my $pkg = ref($_[0]) or return undef;
+	    if (blessed($_[0])) {
+		bless $_[0], 'Scalar::Util::Fake';
+	    }
+	    else {
+		$pkg = undef;
+	    }
+	    "$_[0]" =~ /0x(\w+)/;
+	    my $i = do { local $^W; hex $1 };
+	    bless $_[0], $pkg if defined $pkg;
+	    $i;
+	};
+    }
+}
 
 @EXPORT_OK = qw(_make_elements
 		as_string
@@ -36,7 +62,10 @@ use overload
     '>'		=> \&is_proper_superset,
     '<='	=> \&is_subset,
     '>='	=> \&is_superset,
-    'bool'	=> \&size;
+    'bool'	=> \&size,
+    '@{}'	=> sub { [ $_[0]->members ] },
+    '='         => sub { $_[0]->clone($_[1]) },
+    'cmp'       => sub { "$_[0]" cmp "$_[1]" };
 
 use constant OVERLOAD_BINARY_2ND_ARG  => 1;
 use constant OVERLOAD_BINARY_REVERSED => 2;
@@ -69,7 +98,7 @@ sub new {
 
     my $self = { };
 
-    bless $self, $class;
+    bless $self, ref $class || $class;
 
     $self->_new_hook( \@_ );
 
@@ -160,13 +189,19 @@ sub each {
     return $e;
 }
 
-sub _clone {
+sub _empty_clone {
     my $self     = shift;
     my $original = shift;
 
     $self->{'universe'} = $original->{'universe'};
-
     $self->{'null'    } = $original->{'null'    };
+}
+
+sub _clone {
+    my $self     = shift;
+    my $original = shift;
+
+    $self->_empty_clone($original);
 
     $self->_insert( $original->{'elements'} );
 }
@@ -182,6 +217,15 @@ sub clone {
 
 *copy = \&clone;
 
+sub empty_clone {
+    my $self  = shift;
+    my $clone = (ref $self)->new;
+
+    $clone->_empty_clone( $self );
+
+    return $clone;
+}
+
 sub clear {
     my $self = shift;
 
@@ -203,8 +247,11 @@ sub _union ($$) {
     return ($that->clone,   0, ref $that)
 	if $this->is_null;
 
-    return ($this_universe, 1, ref $this)
-	if $this->is_universal || $that->is_universal;
+    return ($this, 1, ref $this)
+	if $this->is_universal;
+
+    return ($that, 1, ref $that)
+	if $that->is_universal;
 
     my $union = $this->clone;
 
@@ -263,7 +310,7 @@ sub _intersection ($$) {
 
     my %intersection = _make_elements $intersection->elements;
 
-    delete @intersection{ $that->elements };
+    delete @intersection{ keys %{{ _make_elements $that->elements }} };
 
     $intersection->delete( values %intersection );
 
@@ -437,6 +484,91 @@ sub unique {
     return $this->_unique( @_ );
 }
 
+sub _make_cartesian_product_iterator {
+    my @iter;
+    my @value;
+    for my $set (@_) {
+	return unless $set->isa('Set::Scalar');
+	my @member = $set->members;
+	my %member;
+	@member{@member} = @member;
+	push @iter, \%member;
+	push @value, scalar CORE::each(%{ $iter[-1] });
+    }
+    return sub {
+	return unless @iter;
+	my @now = @value;
+	my $ix;
+	for ($ix = $#iter; $ix >= 0; $ix--) {
+	    my $next = CORE::each(%{ $iter[$ix] });
+	    if (defined $next) {
+		$value[$ix] = $next;
+		last;
+	    } else {
+		keys %{ $iter[$ix] };  # Reset the iterator.
+		$value[$ix] = CORE::each(%{ $iter[$ix] });
+	    }
+	}
+	if ($ix < 0) {
+	    @iter = ();  # All done.
+	}
+	return @now;
+    };
+}
+
+sub cartesian_product_iterator {
+    shift unless ref $_[0];
+    return &_make_cartesian_product_iterator;
+}
+
+sub cartesian_product {
+    my $iterator = &cartesian_product_iterator;
+    return unless defined $iterator;
+    my $product = $_[0]->empty_clone;
+    while (my @member = $iterator->()) {
+	$product->insert(\@member);
+    }
+    return $product;
+}
+
+sub _make_power_set_iterator {
+    return unless $_[0]->isa('Set::Scalar');
+    my @member = $_[0]->members; 
+    my @iter   = (0) x @member;
+    return sub {
+	return unless @iter;
+	my $ix;
+	for ($ix = 0; $ix < @iter; $ix++) {
+	    if ($iter[$ix]++ == 0) {
+		last;
+	    } else {
+		$iter[$ix] = 0;
+	    }
+	}
+	if ($ix == @iter) {
+	    @iter = ();  # All done.
+	}
+	return map { $member[$_] } grep { $iter[$_] } 0..$#iter;
+    };
+}
+
+sub power_set_iterator {
+    shift unless ref $_[0];
+    return &_make_power_set_iterator;
+}
+
+sub power_set {
+    my $iterator = &power_set_iterator;
+    return unless defined $iterator;
+    my $power = $_[0]->empty_clone;
+    my @member;
+    do {
+	@member = $iterator->();
+	$power->insert($_[0]->empty_clone->insert(@member));
+    } while (@member);
+    return $power;
+}
+
 sub is_universal {
     my $self = shift;
 
