@@ -3,8 +3,9 @@ package Set::Scalar::Virtual;
 use strict;
 local $^W = 1;
 
-use vars qw(@ISA);
+use vars qw($VERSION @ISA);
 
+$VERSION = '1.29';
 @ISA = qw(Set::Scalar::Base);
 
 use Set::Scalar::Base qw(_make_elements as_string _compare _strval);
